/*
 * vip1710osd.c: 
 *
 * See the README file for copyright information and how to reach the author.
 *
 */

#include "vip1710osd.h"
#include <errno.h>
#include <assert.h>
#include <sys/ioctl.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <vdr/tools.h>
#include "vip1710tools.h"
#include "vip1710device.h"

#if VDRVERSNUM < 10509
cVip1710Osd::cVip1710Osd(int Left, int Top, IDirectFBSurface *dfb_primary)
    :cOsd(Left, Top)
#else
cVip1710Osd::cVip1710Osd(int Left, int Top, uint Level, IDirectFBSurface *dfb_primary)
    :cOsd(Left, Top, Level)
#endif
{
    osd_dfb_primary = dfb_primary;
    shown = false;
}

cVip1710Osd::~cVip1710Osd()
{
#if VDRVERSNUM < 10509
  if (shown) {
    cBitmap *Bitmap;
    for ( int i = 0; ( Bitmap = GetBitmap(i)) != NULL; i++ ) {
        Hide(Bitmap);
        }
    }
#else
    SetActive(false);
#endif
#ifdef SET_VIDEO_WINDOW
    if ( vidWin.bpp != 0 ) {
	    ResetVideoSize();
    }
#endif
}

#if VDRVERSNUM >= 10509
void cVip1710Osd::SetActive(bool On)
{
  if (On != Active()) {
      cBitmap *Bitmap;
      // Clears the OSD screen image when it becomes active
      // removes it from screen when it becomes inactive
      cOsd::SetActive(On);
      if (On) {
          for (int i = 0; (Bitmap = GetBitmap(i)) != NULL; i++) {
              Hide(Bitmap);
              }

          if (GetBitmap(0)) // only flush here if there are already bitmaps
             Flush();

      }
      else if (shown) {
          for (int i = 0; (Bitmap = GetBitmap(i)) != NULL; i++) {
              Hide(Bitmap);
              }
          shown = false;
      }
  }
}
#endif

/* We can handle any number of windows */
eOsdError cVip1710Osd::CanHandleAreas(const tArea *Areas, int NumAreas)
{
    eOsdError Result = cOsd::CanHandleAreas(Areas, NumAreas);

    return Result;
}

eOsdError cVip1710Osd::SetAreas(const tArea *Areas, int NumAreas)
{
  if (shown) {
    cBitmap *Bitmap;
    for ( int i = 0; ( Bitmap = GetBitmap(i)) != NULL; i++ ) {
        Hide(Bitmap);
        }
    shown = false;
    }
  return cOsd::SetAreas(Areas, NumAreas);
}

void cVip1710Osd::Flush(void)
{
#if VDRVERSNUM >= 10509
    if (!Active())
     return;
#endif
    cBitmap *Bitmap;
    unsigned char *osd;
    int pitch;

    DFBCHECK (osd_dfb_primary->Lock(osd_dfb_primary, DSLF_WRITE, (void**) &osd, &pitch));

    for ( int i = 0; ( Bitmap = GetBitmap(i)) != NULL; i++ ) {
        int  x1 = 0, y1 = 0, x2 = 0, y2 = 0;
        if ( Bitmap->Dirty(x1,y2,x2,y2) ) {
            for ( int yp = y1; yp <= y2; yp++ ) {
                for ( int xp = x1; xp <= x2; xp++  ) {
                    tColor c = Bitmap->Color(*Bitmap->Data(xp,yp));

                    osd[((yp + Top() + Bitmap->Y0()) * pitch) + ((xp +  Bitmap->X0() + Left()) * 4) ] = ( c >> 0 ) & 0xFF;
                    osd[((yp + Top() + Bitmap->Y0()) * pitch) + ((xp +  Bitmap->X0() + Left()) * 4) + 1] = ( c >> 8 ) & 0xFF;
                    osd[((yp + Top() + Bitmap->Y0()) * pitch) + ((xp +  Bitmap->X0() + Left()) * 4) + 2] = ( c >> 16 ) & 0xFF;
                    osd[((yp + Top() + Bitmap->Y0()) * pitch) + ((xp +  Bitmap->X0() + Left()) * 4) + 3] = ( c >> 24 ) & 0xFF;              
                }
            }
        }
    }
    DFBCHECK (osd_dfb_primary->Unlock(osd_dfb_primary));
#ifdef SET_VIDEO_WINDOW
    if (vidWin.bpp != 0 ) {
	    SetVideoSize(vidWin.x1,vidWin.y1,vidWin.x2 - vidWin.x1,vidWin.y2 - vidWin.y1);
    } else {
	    ResetVideoSize();
    }
#endif
    shown = true;
}

void cVip1710Osd::Hide(cBitmap *Bitmap)
{
    unsigned char *osd;
    int pitch;

    DFBCHECK (osd_dfb_primary->Lock(osd_dfb_primary, DSLF_WRITE, (void**) &osd, &pitch));
    for ( int yp = 0; yp < Bitmap->Height(); yp++ ) {
        for ( int xp = 0; xp < Bitmap->Width(); xp++ ) {
            osd[((yp + Top() + Bitmap->Y0()) * pitch) + ((xp +  Bitmap->X0() + Left()) * 4) ] = 0;
            osd[((yp + Top() + Bitmap->Y0()) * pitch) + ((xp +  Bitmap->X0() + Left()) * 4) + 1] = 0;
            osd[((yp + Top() + Bitmap->Y0()) * pitch) + ((xp +  Bitmap->X0() + Left()) * 4) + 2] = 0;
            osd[((yp + Top() + Bitmap->Y0()) * pitch) + ((xp +  Bitmap->X0() + Left()) * 4) + 3] = 0;              
        }
    }
    DFBCHECK (osd_dfb_primary->Unlock(osd_dfb_primary));
}


// --- cVip1710OsdProvider -------------------------------------------------------

cVip1710OsdProvider::cVip1710OsdProvider(IDirectFBSurface *dfb_primary)
{
  osd_dfb_primary = dfb_primary;
}

#if VDRVERSNUM < 10509
cOsd *cVip1710OsdProvider::CreateOsd(int Left, int Top)
{
  return new cVip1710Osd(Left, Top, osd_dfb_primary, osdBuf);
}
#else
cOsd *cVip1710OsdProvider::CreateOsd(int Left, int Top, uint Level)
{
  return new cVip1710Osd(Left, Top, Level, osd_dfb_primary);
}
#endif
